/* AMBCONV - program for convering AMBER topology to GROMACS topology */
/* Filip Ryjacek 2002 */

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <stddef.h>
#include <string.h>
#include <math.h>
#include <mcheck.h>

#include "ambconv.h"

/*----------------------------MAIN------------------------------------ */
int main(int argc, char* argv[])
{
s_atop* atop;

/* debug memory allocation */
#ifdef MEMDEBUG
  mtrace();
#endif

conv = 4.18400000;
version = 1.0;

/* read command-line arguments */
read_commandline(argc, argv);

                                  /* read AMBER topology to structure atop */
read_ambertop(atop);

if (invert == 0 && atop->ifpert <= 0) {
  // cannot use invert with non-perturbation run!
  // reset invert flag
  invert = 1;
}

/* read AMBER coordinates in RST format */
read_rst(atop);

// write coordinates in PDB format 
if (pdb == 0) {
  write_pdb(atop);
}

// write first part of TINER topology file and coordinates 
if (tinker == 0) {
  write_tintop(atop);  // FREE ERRORS!
}

if (gromacs == 0 || tinker == 0) {
  // write GROMACS ITP + NB files 
  write_itp(atop);
}

if (gromacs == 0) {
  // write GROMACS coordinates in GRO format 
  write_gro(atop);

  // write GROMACS TOP 
  write_grotop(atop);
}


if (lacc_rst == 0) {
    // write low accuracy RST
    write_lacc_rst(atop);
}

// write TINKER bonded parameters and charges
if (tinker == 0) {
  write_tintop_bon(atop);
  write_tintop_charge(atop); 
  write_xyz(atop);
  write_key(atop);
}

#ifdef MEMDEBUG
  muntrace();
#endif

return(0);
}

