/* AMBGRO - program for convering AMBER topology to GROMACS topology */
/* Filip Ryjacek 2002 */

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <stddef.h>
#include <string.h>
#include <math.h>
#include <mcheck.h>

#include "ambconv.h"


/* write [constraints] to .top */
void write_constraints(s_atop* atop)
  {
    unsigned int i;
    unsigned int num_constraints;
    /* atom number */
    int atom1, atom2, atom3; /* atom4; */
    unsigned int index;
    /* length, angles */
    double bl;
    double av;
/*    double phi; */
    /* misc */
    double cos_av;
    double bl_13;

    num_constraints = 0;
    /* write header */
    fprintf(grotop, "\n [ constraints ]\n");

    /* BONDS */
    if (atop->ifpert > 0)
      {
        for (i=0; i < atop->nbper; i++)
	  {
	    /* lambda=1, non-perturbed */
	    atom1 = atop->ibper[i];
	    atom2 = atop->jbper[i];
	    index = atop->icbper[i];
	    bl = atop->req[index-1];
            /* convert to nm */
            bl = bl*0.1000;
            fprintf(grotop, "  %4d%4d%s%12.5f%12.5f\n", atom1, atom2, "   2", bl, bl);
	    num_constraints++;
	  }
      }
    else /* nonperturbed AMBER topology */
      {
    /* bonds without hydrogen */
    for (i=0; i < atop->mbona; i++)
      {
        atom1 = atop->ib[i];
	atom2 = atop->jb[i];
	index = atop->icb[i];
        /* bond length */
	bl = atop->req[index-1];
        /* convert to nm */
        bl = bl*0.1000;
        fprintf(grotop, "  %4d%4d%s%12.5f\n", atom1, atom2, "   2", bl);
	num_constraints++;
      }
    /* bonds with hydrogen */
    for (i=0; i < atop->nbonh; i++)
      {
        atom1 = atop->ibh[i];
	atom2 = atop->jbh[i];
	index = (atop->icbh[i]);
        /* bond length, A */
	bl = atop->req[index-1];
        /* convert to nm */
        bl = bl*0.1000;
        fprintf(grotop, "  %4d%4d%s%12.5f\n", atom1, atom2, "   2", bl);
	num_constraints++;
      }
    }
    
    fprintf(grotop, ";\n");
    
    /* ANGLES */
    if (atop->ifpert > 0)
      {
        for (i=0; i < atop->ngper; i++)
	  {
	    /* non-perturbed paramesters */
            atom1 = atop->ipter[i];
	    atom2 = atop->jpter[i];
	    atom3 = atop->kpter[i];
	    index = atop->icpter[i];
            /* angle in deg */
	    av  = atop->teq[index-1];
            /* convert to rad&calc. cosine */
	    cos_av = cos(av/57.29577951);
	    bl_13 = dist_angle13(bond_length(atop,atom1,atom2), bond_length(atop,atom3,atom2), cos_av);

            fprintf(grotop, "  %4d%4d%s%12.5f%12.5f\n", atom1, atom3, "   2", bl_13, bl_13);
	    num_constraints++;
	  }
      }
    else
      {
    /* non-perturbed angles */ 
    /* angles without hydrogen */
    for (i=0; i < atop->mtheta; i++)
      {
        atom1 = atop->it[i];
	atom2 = atop->jt[i];
	atom3 = atop->kt[i];
	index = atop->ict[i];
        /* angle in deg */
	av  = atop->teq[index-1];
        /* convert to rad&calc. cosine */
	cos_av = cos(av/57.29577951);
	bl_13 = dist_angle13(bond_length(atop,atom1,atom2), bond_length(atop,atom3,atom2), cos_av);

        fprintf(grotop, "  %4d%4d%s%12.5f\n", atom1, atom3, "   2", bl_13);
	num_constraints++;
      }
    /* angles with hydrogen */
    for (i=0; i < atop->ntheth; i++)
      {
        atom1 = atop->ith[i];
	atom2 = atop->jth[i];
	atom3 = atop->kth[i];
	index = atop->ickh[i];
        /* angle in deg */
	av  = atop->teq[index-1];
        /* convert to rad&calc. cosine */
	cos_av = cos(av/57.29577951);
	bl_13 = dist_angle13(bond_length(atop,atom1,atom2), bond_length(atop,atom3,atom2), cos_av);

        fprintf(grotop, "  %4d%4d%s%12.5f\n", atom1, atom3, "   2", bl_13);
	num_constraints++;
      }
    }
    
    fprintf(grotop, "\n; 3N-6 = %6d\n", (atop->natom)*3-6);
    fprintf(grotop, "; constraints = %6d\n", num_constraints);
    fprintf(grotop, "; rest = %6d\n", ((atop->natom)*3-6) - num_constraints);
    fprintf(grotop, "\n; 1-4 constraints (dihedrals)\n; not implemented!\n");
}
    
/* return bond length between atom1 and atom2 */
double bond_length(s_atop* atop, int atom1, int atom2)
{
    unsigned int index;
    double bl;
    unsigned int i;
    int a;
    int b;

    if (atop->ifpert > 0)
      {
        for (i=0; i < atop->nbper; i++)
	  {
	    a = atop->ibper[i];
	    b = atop->jbper[i];

            if (((a == atom1) && (b == atom2)) || ((a == atom2) && (b == atom1)))
              {
	        index = atop->icbper[i];
	        bl = atop->req[index-1];
                /* convert to nm */
                bl = bl*0.1000;
		return bl;
	      }
	  }
      }
    else /* nonperturbed AMBER topology */
      {
    /* bonds without hydrogen */
    for (i=0; i < atop->mbona; i++)
      {

        a = atop->ib[i];
	b = atop->jb[i];

        if (((a == atom1) && (b == atom2)) || ((a == atom2) && (b == atom1)))
          {
	    index = atop->icb[i];
            /* bond length */
	    bl = atop->req[index-1];
            /* convert to nm */
            bl = bl*0.1000;
            return bl;
	  }	    
      }
    /* bonds with hydrogen */
    for (i=0; i < atop->nbonh; i++)
      {
        a = atop->ibh[i];
	b = atop->jbh[i];

      if (((a == atom1) && (b == atom2)) || ((a == atom2) && (b == atom1)))
        {
	  index = (atop->icbh[i]);
          /* bond length, A */
	  bl = atop->req[index-1];
          /* convert to nm */
          bl = bl*0.1000;
          return bl;
        }
      }
    }
    return 0;
}

/*  */
double dist_angle13(double rab, double rbc, double cosine)
{
  return sqrt((rab*rab)+(rbc*rbc)-2.0*(rab*rbc*cosine));
}
