/* AMBCONV - program for convering AMBER topology to GROMACS/TINKER topology */
/* Filip Ryjacek 2002 */

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <stddef.h>
#include <string.h>
#include <math.h>
#include <mcheck.h>

#include "ambconv.h"

/* write bonded information */
void write_tintop_bon(s_atop* atop)
  {
    unsigned int i;
    unsigned int mtd_flag;
    /* atom number */
    int atom1, atom2, atom3, atom4;
    unsigned int index;
    /* force constants, length, angles */
    double bfc, bl;
    double afc, av;
    double phi, dfc;
    int mult;
    // end group interaction
    unsigned int eg_flag;
    char* eg="     ";
    // atom type
    char* type;
    char* type1;
    char* type2;
    char* type3;
    char* type4;

#ifdef DEBUG_TTOP
  printf("\nwrite_tintop_bon: opening %s...\n", tintop_filename);
#endif
    tintop = open_file(tintop_filename, "a");

    // write header
    fprintf(tintop, "\n# Bond Stretching Parameters\n\n");

    // allocate space for atom types from isymbl    
    type  = (char*) xmalloc(5);       
    type1 = (char*) xmalloc(5);       
    type2 = (char*) xmalloc(5);       
    type3 = (char*) xmalloc(5);       
    type4 = (char*) xmalloc(5);       

    // BONDS
    // bonds without hydrogen
    fprintf(tintop, "# bonds without hydrogen\n");

    for (i=0; i < atop->mbona; i++)
      {
        atom1 = atop->ib[i];
	atom2 = atop->jb[i];
	atom1--;
	atom2--;
	index = atop->icb[i];
        /* force constant & bond length kcal/mol,A */
	bfc = atop->rk[index-1];
	bl = atop->req[index-1];

        /* get atom type */
	type1 = get_slice(atop->isymbl, type1, 4,4, atom1);
	
	type2 = get_slice(atop->isymbl, type2, 4,4, atom2);

        fprintf(tintop, "bond%8d%6d%12.2f%12.4f", atop->iac[atom1], atop->iac[atom2], bfc, bl);
        fprintf(tintop, "%s%4s-%4s\n", "   # ", type1, type2);
      }

    /* bonds with hydrogen */
    fprintf(tintop, "# bonds with hydrogen\n");

    for (i=0; i < atop->nbonh; i++)
      {
        atom1 = atop->ibh[i];
	atom2 = atop->jbh[i];
	atom1--;
	atom2--;
	index = (atop->icbh[i]);
        /* force constant & bond length kcal/mol,A */
	bfc = atop->rk[index-1];
	bl = atop->req[index-1];
        /* get atom type */
	type1 = get_slice(atop->isymbl, type1, 4,4, atom1);
	type2 = get_slice(atop->isymbl, type2, 4,4, atom2);

        fprintf(tintop, "bond%8d%6d%12.2f%12.4f", atop->iac[atom1], atop->iac[atom2], bfc, bl);
        fprintf(tintop, "%s%4s-%4s\n", "   # ", type1, type2);
      }

    // ANGLES
        fprintf(tintop, "\n# Angle Bending Parameters\n\n");

    // angles without hydrogen
    fprintf(tintop,"# angles without hydrogen\n");

    for (i=0; i < atop->mtheta; i++)
      {
        atom1 = atop->it[i];
	atom2 = atop->jt[i];
	atom3 = atop->kt[i];
        atom1--;
	atom2--;
	atom3--;
	index = atop->ict[i];
        // force constant in kcal/mol, deg
	afc = atop->tk[index-1];
	av  = atop->teq[index-1];
        // get atom type
	type1 = get_slice(atop->isymbl, type1, 4,4, atom1);
	type2 = get_slice(atop->isymbl, type2, 4,4, atom2);
	type3 = get_slice(atop->isymbl, type3, 4,4, atom3);

        fprintf(tintop, "angle%8d%6d%6d%12.2f%12.2f", atop->iac[atom1], atop->iac[atom2], atop->iac[atom3],afc, av);
        fprintf(tintop, "%s%4s-%4s-%4s\n", "   # ", type1, type2, type3);
      }
      
    // angles with hydrogen
    fprintf(tintop,"# angles with hydrogen\n");
    for (i=0; i < atop->ntheth; i++)
      {
        atom1 = atop->ith[i];
	atom2 = atop->jth[i];
	atom3 = atop->kth[i];
        atom1--;
	atom2--;
	atom3--;
	index = atop->ickh[i];
        /* force constant in kcal/mol, deg */
	afc = atop->tk[index-1];
	av  = atop->teq[index-1];
        /* get atom type */
	type1 = get_slice(atop->isymbl, type1, 4,4, atom1);
	type2 = get_slice(atop->isymbl, type2, 4,4, atom2);
	type3 = get_slice(atop->isymbl, type3, 4,4, atom3);

        fprintf(tintop, "angle%8d%6d%6d%12.2f%12.2f", atop->iac[atom1], atop->iac[atom2], atop->iac[atom3],afc, av);
        fprintf(tintop, "%s%4s-%4s-%4s\n", "   # ", type1, type2, type3);
      }

    
    
    // DIHEDRALS
    fprintf(tintop, "\n# Torsion Parameters\n\n");

    // dihedrals without hydrogen
    fprintf(tintop,"# dihedrals without hydrogen\n");
    for (i=0; i < atop->mphia; i++)
      {
        atom1 = atop->ip[i];
	atom2 = atop->jp[i];
	atom3 = atop->kp[i];
	atom4 = atop->lp[i];

          if (atom4 < 0) { atom4 = abs(atom4); }

        eg_flag = 0;
          if (atom3 < 0) { eg_flag=1; atom3=abs(atom3);} // end group int.

        atom1--;
	atom2--;
        atom3--;
	atom4--;

        // reset multi-termed dihedral flag
	mtd_flag = 0;
	do { // do while mtd_flag > 0

	index = atop->icp[i]; // get index for i-th dihedral
	
        if (mtd_flag > 0) index = index + mtd_flag; // if multitermed dihedral, increment index
        
	// force constant in kcal/mol, deg
	dfc  = atop->pk[index-1];
	  if (dfc < 0.0001) {
	    dfc = 0.001;
	  }
	mult = atop->pn[index-1];
	phi  = atop->phase[index-1];
	
        // get atom type
	type1 = get_slice(atop->isymbl, type1, 4,4, atom1);
	type2 = get_slice(atop->isymbl, type2, 4,4, atom2);
	type3 = get_slice(atop->isymbl, type3, 4,4, atom3);
	type4 = get_slice(atop->isymbl, type4, 4,4, atom4);

        eg = "  ";
        if (eg_flag == 1) { eg = "eg"; }
        fprintf(tintop, "torsion%10d%4d%4d%4d",atop->iac[atom1],atop->iac[atom2],atop->iac[atom3],atop->iac[atom4]);
	fprintf(tintop, "%12.3f%8.3f%6d", dfc, phi, abs(mult));
        fprintf(tintop, "   # %4s-%4s-%4s-%4s", type1, type2, type3, type4);

    	    if (mult < 0) {
		mtd_flag++;
		atom1--; atom2--; atom3--; atom4--; 
		fprintf(tintop," MTD\n");
	    }
	    else {
		mtd_flag = 0;
		fprintf(tintop,"\n");
	    }
	} while (mtd_flag > 0); // end of do cycle

      }

    // dihedrals with hydrogen
    fprintf(tintop,"# dihedrals with hydrogen\n");

    for (i=0; i < atop->nphih; i++)
      {
        atom1 = atop->iph[i];
	atom2 = atop->jph[i];
	atom3 = atop->kph[i];
	atom4 = atop->lph[i];

          if (atom4 < 0) { atom4 = abs(atom4); }

        eg_flag = 0;
        if (atom3 < 0) { eg_flag=1; atom3=abs(atom3);} // end group int.

        atom1--;
	atom2--;
        atom3--;
	atom4--;

        // reset multi-termed dihedral flag
	mtd_flag = 0;
	do { // do while mtd_flag > 0

	index = atop->icph[i];

        if (mtd_flag > 0) index = index + mtd_flag; // if multitermed dihedral, increment index

        // force constant in kcal/mol, deg
	dfc  = atop->pk[index-1];
	mult = atop->pn[index-1];
	phi  = atop->phase[index-1];
	
        // get atom type
	type1 = get_slice(atop->isymbl, type1, 4,4, atom1);
	type2 = get_slice(atop->isymbl, type2, 4,4, atom2);
	type3 = get_slice(atop->isymbl, type3, 4,4, atom3);
	type4 = get_slice(atop->isymbl, type4, 4,4, atom4);

        eg = "  ";
        if (eg_flag == 1) { eg = "eg"; }
        fprintf(tintop, "torsion%10d%4d%4d%4d",atop->iac[atom1],atop->iac[atom2],atop->iac[atom3],atop->iac[atom4]);
	fprintf(tintop, "%12.3f%8.3f%6d", dfc, phi, abs(mult));
        fprintf(tintop, "   # %4s-%4s-%4s-%4s", type1, type2, type3, type4);

    	    if (mult < 0) {
		mtd_flag++;
		atom1--; atom2--; atom3--; atom4--; 
		fprintf(tintop," MTD\n");
	    }
	    else {
		mtd_flag = 0;
		fprintf(tintop,"\n");
	    }
	} while (mtd_flag > 0); // end of do cycle

      }  // end of 'for' cycle "dihedrals with hydrogens"

    fprintf(tintop, "\n");    
    
    close_file(tintop);
#ifdef DEBUG_TTOP
    printf("write_tintop_bon: %s closed.\n", tintop_filename);    
#endif

#ifdef USE_FREE
free(type);
free(type1);
free(type2);
free(type3);
free(type4);
#endif


  }


